#!/afs/athena/contrib/perl/perl
#
# Author: Gilbert Leung (AVP Fall 94)
# 
# rewritten from old version by Dave Cho
# 

$dir = "REF";
push(@INC, $dir);		# important if you run from another dir

require "ctime.pl";
require ".libdbase.pl";

$prog = &progname();		# name of the program you run

$outfile = "$prog.ps";
$master  = "$dir/labels.info";
$header  = ".head-$prog";
$header_progs = "labels|signup|addrlist"; # programs that need .head-$prog

##################### setup default flags according to the program you run
if    ($prog =~ "signup")   { $def_flags = "e"; }
elsif ($prog =~ "addrlist") { $def_flags = "e-"; }
elsif ($prog =~ "labels")   { $def_flags = "e-"; }
elsif ($prog =~ "athena")   { $def_flags = "e-"; $outfile = "-"; }
elsif ($prog =~ "maintain") { $def_flags = "w-"; $outfile = "$prog.tmp$$"; }

############################## Program starts ##############################
($flags, $outfile) = &parseargv($def_flags, $outfile, @ARGV);
%regex = &statflags($flags);

print STDERR "Output going to ";
if ($outfile eq "-") {
    print STDERR "STDOUT\n";
} else {
    print STDERR "$outfile\n";
    print STDERR "Use \n\tlpr -Pprinter $outfile\nto print it out.\n";
}

open(OUTFILE, ">$outfile") || die "Can't open output file $outfile: $!";
open(RAWDATA, $master)     || die "Can't open master $master: $!";

if ($prog =~ /$header_progs/) {
    $date = &ctime(time);	# This is a hack; the definition of date
    chop($date);		# has to come before the end of the header
    open(HEADERFILE, $header)  || die "Can't open header file $header: $!";
    print OUTFILE "%!\n/gendate ($date) def\n\n"; # file, so I'm prepending it.
    print OUTFILE while(<HEADERFILE>);
    print OUTFILE '%' x 79, "\n";
}

############################## Program main loop ##############################
while (<RAWDATA>) {
    chop;
    next if /^#/;		# Skip comments (# in 1st column)
    next if /^\s*$/;		#  and blank lines
    ($name, $stat, $phone, $addr, $email) = split(/\|/);
    $entry = $_;
    $_ = $stat;
    if    (defined($re=$regex{"brother"}) && /$re/) { push(@brother,$entry); }
    elsif (defined($re=$regex{"pledge"})  && /$re/) { push(@pledge,$entry); }
    elsif (defined($re=$regex{"advisor"}) && /$re/) { push(@advisor,$entry); }
    elsif (defined($re=$regex{"alumnus"}) && /$re/) { push(@alumnus,$entry); }
    elsif (defined($re=$regex{"friend"})  && /$re/) { push(@friend,$entry); }
    elsif (defined($re=$regex{"other"})   && /$re/) { push(@other,$entry); }
}

$e_proc = $prog . "_entry";
$h_proc = $prog . "_header";

do out("BROTHERS", $h_proc, $e_proc, @brother) if (defined($regex{"brother"}));
do out("PLEDGES",  $h_proc, $e_proc, @pledge)  if (defined($regex{"pledge"}));
do out("ADVISORS", $h_proc, $e_proc, @advisor) if (defined($regex{"advisor"}));
do out("ALUMNI",   $h_proc, $e_proc, @alumnus) if (defined($regex{"alumnus"}));
do out("FRIENDS",  $h_proc, $e_proc, @friend)  if (defined($regex{"friend"}));
do out("OTHERS",   $h_proc, $e_proc, @other)   if (defined($regex{"other"}));

############################## Output prolog ##############################
if ($prog =~ /$header_progs/) {
    print OUTFILE "\nshowpage\n";
    close OUTFILE || die "blargh";
}
if ($prog =~ /maintain/) {
    unlink("$master.bak");
    link ($master, "$master.bak") || die;  # mv master master.bak
    unlink ($master) || die;
    link ($outfile, $master) || die;       # mv outfile master
    unlink ($outfile) || die;
}

exit 0;

######################################################################
# General Output Procedure
######################################################################

sub out {
    local(%master);
    local($title) = shift;
    local($header_proc) = shift;
    local($entry_proc)  = shift;
    do $header_proc($title) unless ($#_ le $[);	# print header unless empty list
    while ($_ = shift) {
      ($name, $stat, $phone, $addr, $email) = split(/\|/);
      @names = split(/\s/, $name);
      $firstname = $names[$[];
      $lastname = $names[$#names];
      $master{&make_key($firstname, $lastname)} = $_;
    }
    foreach $key (sort keys(%master)) {
	do $entry_proc($master{$key});
    }
}

sub make_key { # first name, last name
  local($firstname) = shift;
  local($lastname) = shift;

  # canonicalize to lower case
  $firstname =~ y/A-Z/a-z/;
  $lastname =~ y/A-Z/a-z/;

  # Form key by concatenating:
  #  1st 20 characters of the last name (add spaces to fill)
  #  first name
  pack("A20",$lastname).$firstname;
}

######################################################################
# signup.perl procs
######################################################################
sub signup_header {
    print OUTFILE "(        ", $_[$[], ") () go\n";
}
sub signup_entry {
    local($name, $stat, $phone, $addr, $email) = split(/\|/, $_[$[]);
    print OUTFILE "($name) ($phone) go\n";
}

######################################################################
# addrlist.perl procs
######################################################################
sub addrlist_header {
    print OUTFILE "() () () () (        ", shift, ") go\n";
}
sub addrlist_entry {
    local($name, $stat, $phone, $addr, $email) = split(/\|/, $_[$[]);
    ($addr1, $addr2) = split (/;/, $addr);
    if ($addr2 eq "") {
	print OUTFILE "($email) ($phone) ($addr) ($name) go\n";
    } else {
	print OUTFILE "($email) ($phone) ($addr2) ($addr1) ($name) gotwo\n";
    }
}

######################################################################
# labels.perl procs
######################################################################
sub labels_header {		# no header for labels
}
sub labels_entry {
    local($name, $stat, $phone, $addr, $email) = split(/\|/, $_[$[]);

    # suppress printing unknown labels with unknown address
    if ($addr !~ '\?') {
	($addr1, $addr2) = split (/;/, $addr);
	print OUTFILE "($name) ($addr1) ($addr2) go\n";
    }
}

######################################################################
# athena.perl procs
######################################################################
sub athena_header { }
sub athena_entry {
    local($name, $stat, $phone, $addr, $email) = split(/\|/, $_[$[]);
    $_ = $email;
    if (/@mit.edu$/) {
	($x, $y) = split(/[,\s]+/); # for couples : foo@mit.edu, bar@mit.edu
	&username($x) if ($x ne "");
	&username($y) if ($y ne "");
    }
}
sub username {
    local($_) = $_[$[];
    s/@mit.edu$//;
    print OUTFILE "$_\n";
}

######################################################################
# maintain.perl procs
######################################################################
sub maintain_header { 
    print OUTFILE "\n", '#' x 79, "\n";
    print OUTFILE "# $_[$[]\n";
    print OUTFILE '#' x 79, "\n\n";
}    
sub maintain_entry {
    print OUTFILE "$_[$[]\n";
}
